@@ -161,8 +161,10 @@ bru_safe_sp <- function(quietly = FALSE,
   if (sp_version >= "1.6-0") {
     # Default to 2L to allow future sp to stop supporting
     # get_evolution_status; assume everything is fine if it fails.
-    evolution_status <- tryCatch(sp::get_evolution_status(),
-      error = function(e) 2L
+    evolution_status <- tryCatch(
+      sp::get_evolution_status(),
+      error = function(e) 2L,
+      warning = function(e) 2L
     )
     rgdal_version <- tryCatch(utils::packageVersion("rgdal"),
       error = function(e) NA_character_
@@ -378,6 +380,7 @@ eval_spatial_Spatial <- function(data, where, layer = NULL, selector = NULL) {
     )
   ))
   if (inherits(where, "SpatialPoints")) {
+    where <- fm_transform(where, crs = fm_CRS(data), passthrough = TRUE)
     if (ncol(sp::coordinates(where)) >= 3) {
       where <- sp::SpatialPoints(
         coords = sp::coordinates(where)[, 1:2, drop = FALSE],
@@ -410,9 +413,11 @@ eval_spatial_Spatial <- function(data, where, layer = NULL, selector = NULL) {
 #' @describeIn eval_spatial Supports point-in-polygon information lookup.
 #' Other combinations are untested.
 eval_spatial.sf <- function(data, where, layer = NULL, selector = NULL) {
-  if (inherits(where, "SpatialPoints")) {
+  if (!inherits(where, c("sf", "sfc", "sfg"))) {
     where <- sf::st_as_sf(where)
   }
+  where <- fm_transform(where, crs = sf::st_crs(data), passthrough = TRUE)
+
   layer <- extract_layer(where, layer, selector)
   check_layer(data, where, layer)
   unique_layer <- unique(layer)
@@ -461,6 +466,7 @@ eval_spatial.SpatRaster <- function(data, where, layer = NULL, selector = NULL)
   if (!inherits(where, "SpatVector")) {
     where <- terra::vect(where)
   }
+  where <- terra::project(where, data)
   if ((NROW(where) == 1) && (terra::nlyr(data) > 1)) {
     # Work around issue in terra::extract() that assumes `layer` to point
     # to a column of `where` (like `selector`) when
@@ -533,7 +539,8 @@ eval_spatial.stars <- function(data, where, layer = NULL, selector = NULL) {
 #' `TRUE`
 #' @param layer,selector Specifies what data column or columns from which to
 #' extract data, see [component()] for details.
-#' @param batch_size Size of nearest-neighbour calculation blocks, to limit the
+#' @param batch_size `r lifecycle::badge("deprecated")` due to improved algorithm.
+#' Size of nearest-neighbour calculation blocks, to limit the
 #' memory and computational complexity.
 #' @return An infilled vector of values
 #' @export
@@ -563,7 +570,7 @@ eval_spatial.stars <- function(data, where, layer = NULL, selector = NULL) {
 #' }
 bru_fill_missing <- function(data, where, values,
                              layer = NULL, selector = NULL,
-                             batch_size = 50) {
+                             batch_size = deprecated()) {
   stopifnot(inherits(
     data,
     c(
@@ -615,8 +622,12 @@ bru_fill_missing <- function(data, where, values,
 
   data_crs <- fm_crs(data)
   if (inherits(data, "SpatRaster")) {
-    data_values <- terra::values(data[[layer]], dataframe = TRUE)[[layer]]
-    data_coord <- as.data.frame(terra::crds(data))
+    data_values <- terra::values(
+      data[[layer]],
+      dataframe = TRUE,
+      na.rm = TRUE
+    )[[layer]]
+    data_coord <- as.data.frame(terra::crds(data[[layer]], na.rm = TRUE))
     data_coord <- sf::st_as_sf(data_coord,
       coords = seq_len(ncol(data_coord)),
       crs = data_crs
@@ -628,27 +639,15 @@ bru_fill_missing <- function(data, where, values,
 
   where <- fm_transform(where, crs = data_crs, passthrough = TRUE)
 
-  notok <- is.na(values)
-  ok <- which(!notok)
-  notok <- which(notok)
-  data_notok <- is.na(data_values)
-  data_ok <- which(!data_notok)
-  data_notok <- which(data_notok)
+  values_notok <- is.na(values)
 
-  for (batch in seq_len(ceiling(length(notok) / batch_size))) {
-    subset <- notok[seq((batch - 1) * batch_size,
-      min(length(notok), batch * batch_size),
-      by = 1
-    )]
-    dst <- sf::st_distance(
-      data_coord[data_ok, , drop = FALSE],
-      where[subset, , drop = FALSE],
-      by_element = FALSE
-    )
+  nn <- sf::st_nearest_feature(
+    where[values_notok, , drop = FALSE],
+    data_coord
+  )
+
+  values[values_notok] <- data_values[nn]
 
-    nn <- apply(dst, MARGIN = 2, function(col) which.min(col)[[1]])
-    values[subset] <- data_values[data_ok[nn]]
-  }
   values
 }
 
@@ -731,7 +730,7 @@ resave_package_data <- function() {
 #' @title Row-wise Kronecker products
 #'
 #' @description
-#' `r lifecycle::badge("deprecated") in favour of [fmesher::fm_row_kron()].
+#' `r lifecycle::badge("deprecated")` in favour of [fmesher::fm_row_kron()].
 #'
 #' Takes two Matrices and computes the row-wise Kronecker product.  Optionally
 #' applies row-wise weights and/or applies an additional 0/1 row-wise Kronecker
