@@ -1,7 +1,8 @@
 \donttest{
 if (bru_safe_inla() &&
     require(ggplot2, quietly = TRUE) &&
-    bru_safe_sp()) {
+    bru_safe_sp() &&
+    require("sp")) {
 
   # Load Gorilla data
 
